//  Created by dasdom on 16.11.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import LogStore
import CoreLocation
import MapKit

class DrawViewController: UIViewController {
  
  private var locationProvider: LocationProvider? = nil
  private var locations: [CLLocation] = []
  private var contentView: DrawView {
    view as! DrawView
  }
  
  override func loadView() {
    let contentView = DrawView(frame: UIScreen.main.bounds)
    
    contentView.startStopButton.addTarget(
      self,
      action: #selector(startStop(_:)),
      for: .touchUpInside)
    
    contentView.clearButton.addTarget(
      self,
      action: #selector(clear(_:)),
      for: .touchUpInside)

    contentView.shareButton.addTarget(
      self,
      action: #selector(share(_:)),
      for: .touchUpInside)

    view = contentView
  }
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    /*
    locationProvider = LocationProvider(
      updateHandler: { location, error in
      
        printLog("location: \(String(describing: location))")
    })
    */
    locationProvider = LocationProvider(
      updateHandler: { [unowned self] location, error in
      
        guard let location = location else { return }
        printLog("location: \(location)")
        self.locations.append(location)

        (self.view as? DrawView)?.addOverlay(with: self.locations)
    })
  }
  
  @objc func startStop(_ sender: UIButton) {
    /*
    locationProvider?.start()
    */
    if locationProvider?.updating ?? false {
      locationProvider?.stop()
      sender.setTitle("Start", for: .normal)
    } else {
      locationProvider?.start()
      sender.setTitle("Stop", for: .normal)
    }
  }
  
  @objc func clear(_ sender: UIButton) {
    locations.removeAll()
    /*
    (view as? DrawView)?.addOverlay(with: locations)
    */
    contentView.addOverlay(with: locations)
  }
  
  @objc func share(_ sender: UIButton) {
    let options = MKMapSnapshotter.Options()
    options.region = contentView.mapView.region
    
    let snapshotter = MKMapSnapshotter(options: options)
    snapshotter.start { snapshot, error in
      guard let snapshot = snapshot else {
        return
      }
      let image = self.imageByAddingPath(
        with: self.locations,
        to: snapshot)
      
      let activity = UIActivityViewController(
        activityItems: [image, "#move2draw"],
        applicationActivities: nil)
      self.present(activity, animated: true, completion: nil)
    }
  }
  
  func imageByAddingPath(
    with locations: [CLLocation],
    to snapshot: MKMapSnapshotter.Snapshot)
    -> UIImage {
      
      UIGraphicsBeginImageContextWithOptions(
        snapshot.image.size, true, 0.0)
      
      snapshot.image.draw(at: CGPoint(x: 0, y: 0))
      
      let bezierPath = UIBezierPath()
      guard let firstCoordinate = locations.first?.coordinate else {
        fatalError()
      }
      let firstPoint = snapshot.point(for: firstCoordinate)
      bezierPath.move(to: firstPoint)
      
      for location in locations.dropFirst() {
        let point = snapshot.point(for: location.coordinate)
        bezierPath.addLine(to: point)
      }
      
      UIColor.red.setStroke()
      bezierPath.lineWidth = 2
      bezierPath.stroke()
      
      guard let image = UIGraphicsGetImageFromCurrentImageContext() else {
        fatalError()
      }
      UIGraphicsEndImageContext()
      
      return image
  }
}
