//  Created by dasdom on 10.12.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import CoreMotion
import LogStore

class MeasurementViewController: UIViewController {

  let motionManager = CMMotionManager()
  /*
  var xAccelerationData: [AccelerationData] = []
  */
  var xAccelerationData: [AccelerationData] = [] {
    didSet {
      xDiagramView.dataArray = xAccelerationData
    }
  }
  var yAccelerationData: [AccelerationData] = [] {
    didSet {
      yDiagramView.dataArray = yAccelerationData
    }
  }
  var zAccelerationData: [AccelerationData] = [] {
    didSet {
      zDiagramView.dataArray = zAccelerationData
    }
  }

  @IBOutlet var xDiagramView: DiagramView!
  @IBOutlet var yDiagramView: DiagramView!
  @IBOutlet var zDiagramView: DiagramView!
  @IBOutlet var resultLabel: UILabel!
  @IBOutlet var lowerBoundSlider: UISlider!
  @IBOutlet var upperBoundSlider: UISlider!

  // ...
  // methods
  // ...

  override func viewDidLoad() {
    super.viewDidLoad()
    // Do any additional setup after loading the view.
    
    /*
    startMotionUpdates()
    */
  }
  
  func startMotionUpdates() {
    
    motionManager.deviceMotionUpdateInterval = 1 / 60
    
    xAccelerationData = []
    yAccelerationData = []
    zAccelerationData = []

    motionManager.startDeviceMotionUpdates(
      using: .xArbitraryZVertical,
      to: OperationQueue()) { motion, error in
        
        guard let motion = motion else {
          return
        }
        print("motion: \(motion)")
        
        let acceleration = motion.userAcceleration
        let timestamp = motion.timestamp
        let xData = AccelerationData(timestamp: timestamp,
                                     value: acceleration.x)
        let yData = AccelerationData(timestamp: timestamp,
                                     value: acceleration.y)
        let zData = AccelerationData(timestamp: timestamp,
                                     value: acceleration.z)
        
        DispatchQueue.main.async {
          self.xAccelerationData.append(xData)
          self.yAccelerationData.append(yData)
          self.zAccelerationData.append(zData)
        }
    }
  }

  func stopMotionUpdates() {
    motionManager.stopDeviceMotionUpdates()
  }

  @IBAction func startStop(_ sender: UIButton) {
    /*
    startMotionUpdates()
    */
    if motionManager.isDeviceMotionActive {
      sender.setTitle("Start", for: .normal)
      stopMotionUpdates()
    } else {
      sender.setTitle("Stop", for: .normal)
      startMotionUpdates()
    }
  }
  
  @IBAction func sliderChanged(_ sender: UISlider) {
    let value = CGFloat(sender.value)
    if sender == lowerBoundSlider {
      xDiagramView.lineOnePosition = value
      yDiagramView.lineOnePosition = value
      zDiagramView.lineOnePosition = value
    } else if sender == upperBoundSlider {
      xDiagramView.lineTwoPosition = value
      yDiagramView.lineTwoPosition = value
      zDiagramView.lineTwoPosition = value
    }
    
    updateResultLabel()
  }
  
  private func periodFromMarkers() -> TimeInterval {
    
    guard let firstTimestamp = xAccelerationData.first?.timestamp,
      let lastTimestamp = xAccelerationData.last?.timestamp else {
        
        return 0
    }

    let totalTime = lastTimestamp - firstTimestamp
    let x1 = xDiagramView.lineOnePosition
    let x2 = xDiagramView.lineTwoPosition
    return TimeInterval(x2 - x1) * totalTime
  }
  
  private func lengthFrom(period: Double) -> Double {
    return 9.81 * pow(period / (2 * Double.pi), 2)
  }
  
  private func updateResultLabel() {
    
    let period = periodFromMarkers()
    let length = lengthFrom(period: period)
    
    resultLabel.text = String(format: "Length: %.3lf m", length)
  }
}

